; decode_recode by evilpaul of atebit
; 
; recode of the digital identity for the v&a decode exhibition
; this version is 512bytes long and runs on the zx spectrum computer
; 
; compile code using pasmo: pasmo --tapbas --alocal decode.asm decode.tap
; 
; code is freely distributed - do with it as you will
; 
; http://www.evilpaul.org (evilpaul@evilpaul.org)
; http://www.atebit.org
; http://www.vam.ac.uk/microsites/decode/
; http://www.arrakis.es/~ninsesabe/pasmo/



PROG_START		equ	32768
GRADIENT_DATA	equ	PROG_START+6*1024
ATTR_BUFFER		equ	GRADIENT_DATA+4*1024


org		PROG_START

	
start	; blank screen and show info message
		call	blankScreen
		ld		de,text
		ld		c,textEnd-text
		call	0x203c

		; create gradient data
		ld		de,GRADIENT_DATA
		ld		hl,16384+2048*3+32*23
		exx
		ld		c,64
_allRows
		ld		b,64
_oneRow
		push	bc
		ld		a,b
		sub		32
		jr		nc,_xpos
		neg
_xpos	call	sqr
		push	hl
		ld		a,c
		sub		32
		jr		nc,_ypos
		neg
_ypos	call	sqr
		pop		de
		add		hl,de
		add		hl,hl
		add		hl,hl
		call	sqrt16
		exx
		add		a,30
		ld		(de),a
		inc		de
		exx
		pop		bc
		djnz	_oneRow
		ld		a,c
		and		1
		jr		nz,_skipBarDrawing
		exx
		ld		(hl),%010010
		inc		l
		push	hl
_fill	jr		z,_filled
		ld		(hl),a
		inc		l
		jr		_fill
_filled	pop		hl
		exx
_skipBarDrawing
		dec		c
		jr		nz,_allRows

		call	blankScreen

		; draw triangle overlay
drawTriangles
		ld		hl,0
		ld		de,16384
		ld		c,3
_oneBlock
		ld		b,32*8
_oneRow
		ld		a,(hl)
		inc		hl
		push	hl
		ld		hl,triangleGfx
		and		%1000
		add		a,l
		ld		l,a
		push	bc
		ld		c,d
		ld		b,8
_oneChar
		ld		a,(hl)
		ld		(de),a
		inc		hl
		inc		d
		djnz	_oneChar
		ld		d,c
		pop		bc
		inc		e
		pop		hl
		djnz	_oneRow
		dec		e
		ld		a,d
		add		a,8
		ld		d,a
		dec		c
		jr		nz,_oneBlock


		; highlight logo
		ld		hl,16384+2048
		ld		de,logo
		ld		b,4*7
_allLogoBytes
		push	bc
		ld		a,(de)
		inc		de
		ld		b,8
_oneLogoByte
		rlca
		jr		nc,_notFlipped
		ex		af,af'
		push	bc
		push	hl
		xor		a
		ld		b,8		
_flipOneChar
		ld		(hl),a
		inc		h
		djnz	_flipOneChar
		pop		hl
		pop		bc
		ex		af,af'
_notFlipped
		inc		hl
		djnz	_oneLogoByte
		pop		bc
		djnz	_allLogoBytes


		; main loop starts here
mainLoop
		di

		; clear screen buffer
		ld		(_restSp+1),sp
		ld		sp,ATTR_BUFFER+32*24
		ld		hl,%0011111100111111
		ld		b,32*24/4
_clear	push	hl
		push	hl
		djnz	_clear
_restSp	ld		sp,1234

		; render all ripples
		ld		ix,rippleData
		ld		de,5
		ld		b,6
_allRipples
		exx
		ld		a,(ix+2)
		and		%10000000
		jr		z,_ok

_read	ld		hl,0
		ld		a,(hl)			; speed
		inc		hl
		and		%1
		inc		a
		ld		(ix+3),a
		ld		a,(hl)			; x
		inc		hl
		and		%11111
		ld		e,a
		ld		a,(hl)			; y
		inc		hl
		and		%11111
		ld		d,a
		inc		hl
		ld		a,h
		and		%11
		ld		h,a
		ld		(_read+1),hl
		ld		h,0
		ld		l,d
		add		hl,hl
		add		hl,hl
		add		hl,hl
		add		hl,hl
		add		hl,hl
		add		hl,hl
		ld		d,0
		add		hl,de
		ld		de,GRADIENT_DATA
		add		hl,de
		ld		(ix+0),l
		ld		(ix+1),h

_ok		ld		a,(ix+2)
		add		a,(ix+3)
		ld		(ix+2),a
		ld		l,(ix+0)
		ld		h,(ix+1)
		ld		b,(ix+4)
		call	drawRipple
		exx
		add		ix,de
		djnz	_allRipples

		; sync and copy to screen
		ei
		halt
		ld		hl,ATTR_BUFFER
		ld		de,16384+2048*3
		ld		a,32*24/6
_copy	ldi
		ldi
		ldi
		ldi
		ldi
		ldi
		dec		a
		jr		nz,_copy

		; repeat forever..
		jp		mainLoop
		

		; draw a single ripple
drawRipple
		ld		(__val1+1),a
		add		a,30
		ld		(__val2+1),a
		ld		a,b
		ld		(__col+1),a

		ld		de,ATTR_BUFFER
		ld		c,24
_allRows
		ld		b,32
_oneRow
		ld		a,(hl)
		inc		l
__val1	cp		123
		jr		c,_off
__val2	cp		123
		jr		nc,_off
		ld		a,(de)
__col	and		123
		ld		(de),a
_off	inc		e
		djnz	_oneRow
		dec		e
		inc		de
		dec		l
		ld		a,64-32+1
		add		a,l
		ld		l,a
		ld		a,b
		adc		a,h
		ld		h,a
		dec		c
		jr		nz,_allRows
		ret

; in:	a = squaree
; out:	hl
sqr		ld		h,a
		ld		e,a
		ld		l,0
		ld		d,l
;		jp		mul8x8

; in:	hl*de (l and d must be 0)
; out:	hl
mul8x8	sla		h
		jr		nc,$+3
		ld		l,e
		ld		b,7
_mul	add		hl,hl
		jr		nc,$+3
		add		hl,de
		djnz	_mul
		ret

; in:	hl = radicand
; out:	a = radix
sqrt16	ld		a,-1
		ld		d,a
		ld		e,a
_find	add		hl,de
		inc		a
		dec		e
		dec		de
		jp		c,_find
		ret


blankScreen
		ld		hl,16384+2048*3
		ld		(hl),%111111
		ld		de,16384+2048*3+1
		ld		bc,32*24-1
		ldir
		ret


triangleGfx
		db		%11111111
		db		%11111110
		db		%11111100
		db		%11111000
		db		%11110000
		db		%11100000
		db		%11000000
		db		%10000000

		db		%11111111
		db		%11101110
		db		%11001100
		db		%10001000
		db		%11111111
		db		%11101110
		db		%11001100
		db		%10001000


rippleData
		dw		0
		db		143,0
		db		%011111
		dw		0
		db		164,0
		db		%101111
		dw		0
		db		185,0
		db		%110111
		dw		0
		db		206,0
		db		%111011
		dw		0
		db		227,0
		db		%111101
		dw		0
		db		248,0
		db		%111110


logo	db		%00110000,%10000100,%00100011,%00001000
		db		%00101001,%01001010,%01010010,%10010100
		db		%00101001,%00001000,%01010010,%10010000
		db		%00101001,%10001000,%01010010,%10011000
		db		%00101001,%00001000,%01010010,%10010000
		db		%00101001,%01001010,%01010010,%10010100
		db		%00110000,%10000100,%00100011,%00001000


text	db		0x16,20,0,"decode_recode",13,"evilpaul@evilpaul.org"
textEnd


end start